# -*- coding: utf-8 -*-
"""
Tests the "trial" flow for every application (from home page to new
instance's app switcher).

Requires:
    ``pytest-selenium``

Usage:
    ``pytest --driver <browser> test/test_trial_flow.py``

    ``<browser>`` can be any browser supported by pytest-selenium
    (http://pytest-selenium.readthedocs.io/en/latest/user_guide.html#specifying-a-browser)

    Specific browsers may require configuration or intermediate tools (e.g.
    chromedriver, geckodriver), see pytest-selenium documentation for details.

Advanced Usage:
    Specific drivers can be further configured by overriding *fixtures*:
    create a local ``conftest.py`` (uncommitted at the root of internal) and
    redefine the documented fixture (available for Firefox, Chrome, PhantomJS
    and capabilities). For instance to configure firefox as headless::

        import pytest

        @pytest.fixture
        def firefox_options(firefox_options):
              firefox_options.add_argument('-headless')
              return firefox_options

Notes:
    * Only tested on next.odoo.com, theoretically could work on test or www
    * Sales and Invoicing currently disabled (installing them does not put an
      application on the application switcher)
    * Only takes a screenshot when the application switcher check fails, not
      on other errors
"""
import os
import tempfile
import uuid

import pytest
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.expected_conditions import \
    presence_of_element_located, text_to_be_present_in_element, \
    element_selection_state_to_be, visibility_of_element_located, staleness_of
from selenium.webdriver.support.select import Select
from selenium.webdriver.support.wait import WebDriverWait

# normal page browsing
PAGE_TIMEOUT = 10
# installing a trial application
INSTALL_TIMEOUT = 120
# download + load webclient & display app switcher may take quite a bit
# longer than a regular page load
APP_SWITCHER_LOAD_TIMEOUT = 30

def app(trialname, appname=None):
    if not appname:
        appname = trialname
    return trialname, (By.XPATH, '//a[div[text() = "%s"]]' % appname)
@pytest.mark.slow
@pytest.mark.parametrize('to_install,check_selector', [
    app("CRM"),
    app("Point of Sale"),
    # TODO: multi-saas handling?
    # app("Sales"), # needs to install sale_management in SAAS-17?
    app("eSign", "Sign"),
    app("Project"),
    app("Inventory"),
    app("Manufacturing"),
    app("Timesheets"),
    app("Website"),
    app("eCommerce", "Website"),
    # app("Invoicing"), # cf sales no app appears
    app("Accounting"),
    app("Mass Mailing", "Email Marketing"),
    app("Events"),
    app("Studio", "New App"), # client is opened with studio launched and a New App placeholder
])
def test_trial_flow(base_url, selenium, to_install, check_selector):
    selenium.get(base_url)

    trial_link = selenium.find_element(By.CSS_SELECTOR, '.o_header_buttons .btn-primary')
    assert trial_link.get_attribute('href').endswith('/trial')
    trial_link.click()

    WebDriverWait(selenium, PAGE_TIMEOUT).until(
        text_to_be_present_in_element((By.TAG_NAME, 'h1'), 'Choose your first App'))
    assert selenium.title == 'Odoo Online Trial | Odoo'
    # link_text is apparently only direct child?
    app_install_link = selenium.find_element(By.CSS_SELECTOR, '.s_appssection_box [alt="%s"]' % to_install)
    app_install_link.click()
    # JS transition, not actual page change
    # wait for form to be loaded
    WebDriverWait(selenium, PAGE_TIMEOUT).until(
        visibility_of_element_located((By.CSS_SELECTOR, 'div.fill-info-step h5')))
    WebDriverWait(selenium, 5).until(visibility_of_element_located((By.NAME, 'username')))

    # fill form, need prefix because "Special characters are not allowed in
    # domain names" (a leading number is apparently a special character)
    text = 'x' + uuid.uuid4().hex

    field = next(e for e in selenium.find_elements(By.TAG_NAME, 'input') if e.is_displayed())
    field.click() # focus first visible input

    fill(selenium, 'username', text)

    fill(selenium, 'email', '%s@mailinator.com' % text)

    fill(selenium, 'company_name', text)

    company_size = selenium.find_element(By.NAME, 'company_size')
    Select(company_size).select_by_visible_text('< 5 employees')

    plan = selenium.find_element(By.NAME, 'plan')
    Select(plan).select_by_visible_text('I am a teacher or a student')

    # wanted to scroll to the submit button and click on it, but I can't get
    # it to work reliably so fuckitjs
    field.submit()

    # wait for load/install page
    loading = WebDriverWait(selenium, PAGE_TIMEOUT).until(
        presence_of_element_located((By.CSS_SELECTOR, 'h3.text-alpha')))
    # then to be redirected from it
    WebDriverWait(selenium, INSTALL_TIMEOUT).until(staleness_of(loading))
    # and make sure the application switcher is loaded
    WebDriverWait(selenium, APP_SWITCHER_LOAD_TIMEOUT).until(
        presence_of_element_located((By.CLASS_NAME, 'o_application_switcher')))
    # then check that the application we expect is present
    if not selenium.find_elements(*check_selector):
        (fd, f) = tempfile.mkstemp(suffix='.png')
        os.close(fd)
        selenium.get_screenshot_as_file(f)
        pytest.fail("did not find expected element in final page, stored screenshot at %s" % f, pytrace=False)
def fill(driver, name, text):
    # Ideally we'd want to select the currently active element to check if
    # it's the one we were expecting and fill it, but that apparently does not
    # work in the safari driver so...
    # field = driver.switch_to.active_element
    field = driver.find_element(By.NAME, name)
    # ensure element is visible (...)
    field.location_once_scrolled_into_view
    field.send_keys(text)
    field.send_keys(Keys.TAB)
    WebDriverWait(driver, 1).until(element_selection_state_to_be(field, False))
